-- PalletSES class
-- Adds the possibility to define a sowing machine trigger and a dynamic mount trigger similar to the pallet and bale objects
-- Requires the Mountable specialization to be present (for dynamic mounting functionality)
--
-- @author  Stefan Geiger
-- @date  25/02/13
--
-- @Edit: Bluebaby210: to fill seeds and bulid it in trailer
-- @date  31.07.2013
--
-- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.

SeedsFill = {};

function SeedsFill.prerequisitesPresent(specializations)
    return true;
end;

function SeedsFill:load(xmlFile)
    local sowingMachineFillTriggerId = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.sowingMachineFillTrigger#index"));
    if sowingMachineFillTriggerId ~= nil then
        self.sowingMachineFillTrigger = SeedsFillTrigger:new(sowingMachineFillTriggerId);
    end;
end;

function SeedsFill:delete()
    if self.sowingMachineFillTrigger ~= nil then
        self.sowingMachineFillTrigger:delete();
    end;
end;

function SeedsFill:mouseEvent(posX, posY, isDown, isUp, button)
end;

function SeedsFill:keyEvent(unicode, sym, modifier, isDown)
end;

function SeedsFill:update(dt)
end;

function SeedsFill:updateTick(dt)
end;

function SeedsFill:draw()
end;

function SeedsFill:dynamicMountTriggerCallback(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
end;

-- PalletSESFillTrigger class
-- Replacement for the original SowingMachineFillTrigger class which has a support lower price and supports dynamic deleting

SeedsFillTrigger = {}

local SeedsFillTrigger_mt = Class(SeedsFillTrigger);

function SeedsFillTrigger:new(nodeId, mt)
    if mt == nil then
        mt = SeedsFillTrigger_mt;
    end
    local self = {};
    setmetatable(self, mt);

    self.triggerId = nodeId;
    addTrigger(nodeId, "triggerCallback", self);
	
	self.neededFruitType = getUserAttribute(nodeId, "neededFruitType"); 
    self.triggeredSowingMachines = {};

    return self;
end;

function SeedsFillTrigger:delete()
    removeTrigger(self.triggerId);

    for sowingMachine,_ in pairs(self.triggeredSowingMachines) do
        if not sowingMachine.isDeleted then
            sowingMachine:removeSowingMachineFillTrigger(self);
        end
    end
end;

function SeedsFillTrigger:triggerCallback(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
    if onEnter or onLeave then
        local sowingMachine = g_currentMission.objectToTrailer[otherShapeId];

        if sowingMachine ~= nil and sowingMachine.addSowingMachineFillTrigger ~= nil and sowingMachine.removeSowingMachineFillTrigger ~= nil then
            if onEnter then
                if self.triggeredSowingMachines[sowingMachine] == nil then
					-- make sure the sowing machine supports the needed fruit type
					local fruitTypeDesc = FruitUtil.fruitTypes[self.neededFruitType];
					if sowingMachine.seeds ~= nil then
								self.triggeredSowingMachines[sowingMachine] = sowingMachine;
								sowingMachine:addSowingMachineFillTrigger(self);
					end;
                end
            else -- onLeave
				if self.triggeredSowingMachines[sowingMachine] ~= nil then
					sowingMachine:removeSowingMachineFillTrigger(self);
					self.triggeredSowingMachines[sowingMachine] = nil;
				end;
            end;
        end;
    end;
end;

function SeedsFillTrigger:fillSowingMachine(sowingMachine, delta)

    local fillType = Fillable.FILLTYPE_SEEDS;
    local priceScale = 0.9;
    if not sowingMachine:allowFillType(fillType, false) then
        return 0.0;
    end
    local oldFillLevel = sowingMachine:getFillLevel(fillType);
    sowingMachine:setFillLevel(oldFillLevel + delta, fillType, true);
    delta = sowingMachine:getFillLevel(fillType) - oldFillLevel;

    local fillTypeDesc = Fillable.fillTypeIndexToDesc[fillType]
    if fillTypeDesc ~= nil then

        local price = delta*fillTypeDesc.pricePerLiter*priceScale;
        g_currentMission.missionStats.expensesTotal = g_currentMission.missionStats.expensesTotal + price;
        g_currentMission.missionStats.expensesSession = g_currentMission.missionStats.expensesSession + price;

        g_currentMission:addSharedMoney(-price, "other");
    end

    return delta;
end

function SeedsFillTrigger:getIsActivatable(fillable)
    if not fillable:allowFillType(Fillable.FILLTYPE_SEEDS, false) then
        return false;
    end;
    return true;
end;